@@ -47,6 +47,8 @@ CAlphabet::CAlphabet(const CAlphIO::AlphInfo &AlphInfo)
   m_StartConversionSymbol = -1;
   m_EndConversionSymbol = -1;
 
+  m_iResetSymbol = -1;
+
   m_strDefaultContext = AlphInfo.m_strDefaultContext;
 
   // Set miscellaneous options
@@ -61,6 +63,8 @@ CAlphabet::CAlphabet(const CAlphIO::AlphInfo &AlphInfo)
     AddChar(it->Text, it->Display, it->Colour, it->Foreground);
 
 
+  // TODO: Special characters are a mess - really need to think these through
+
   // Set Space character if requested
 
   // This line makes it a bit easier for our WindowsCE compiler
@@ -88,6 +92,11 @@ CAlphabet::CAlphabet(const CAlphIO::AlphInfo &AlphInfo)
   if(AlphInfo.ControlCharacter.Display != std::string("") && GetControlSymbol() == -1)
     AddControlSymbol(AlphInfo.ControlCharacter.Text, AlphInfo.ControlCharacter.Display, AlphInfo.ControlCharacter.Colour, AlphInfo.ControlCharacter.Foreground);
 
+  if(AlphInfo.ResetCharacter.Text != empty)
+    AddResetSymbol(AlphInfo.ResetCharacter.Text, AlphInfo.ResetCharacter.Display, 
+		   AlphInfo.ResetCharacter.Colour, AlphInfo.ResetCharacter.Foreground);
+
+
   // New group stuff
 
   m_pBaseGroup = AlphInfo.m_pBaseGroup;
@@ -99,7 +108,7 @@ CAlphabet::CAlphabet(const CAlphIO::AlphInfo &AlphInfo)
 
 /////////////////////////////////////////////////////////////////////////////
 
-void CAlphabet::GetSymbols(vector <symbol >*Symbols, string *Input, bool IsMore) const {
+void CAlphabet::GetSymbols(std::vector<symbol > *Symbols, std::string *Input, bool IsMore) const {
   string Tmp;
   symbol CurSymbol = 0, TmpSymbol = 0;
   bool KeyIsPrefix = false;
@@ -167,7 +176,7 @@ void CAlphabet::GetSymbols(vector <symbol >*Symbols, string *Input, bool IsMore)
 }
 
 // add single char to the character set
-void CAlphabet::AddChar(const string NewCharacter, const string Display, int Colour, const string Foreground) {
+void CAlphabet::AddChar(const std::string NewCharacter, const std::string Display, int Colour, const std::string Foreground) {
   m_Characters.push_back(NewCharacter);
   m_Display.push_back(Display);
   m_Colours.push_back(Colour);
@@ -179,39 +188,45 @@ void CAlphabet::AddChar(const string NewCharacter, const string Display, int Col
 
 /////////////////////////////////////////////////////////////////////////////
 
-void CAlphabet::AddParagraphSymbol(const string NewCharacter, const string Display, int Colour, const string Foreground) {
+void CAlphabet::AddParagraphSymbol(const std::string NewCharacter, const std::string Display, int Colour, const std::string Foreground) {
   AddChar(NewCharacter, Display, Colour, Foreground);
   m_ParagraphSymbol = GetNumberSymbols() - 1;
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
-void CAlphabet::AddSpaceSymbol(const string NewCharacter, const string Display, int Colour, const string Foreground) {
+void CAlphabet::AddSpaceSymbol(const std::string NewCharacter, const std::string Display, int Colour, const std::string Foreground) {
   AddChar(NewCharacter, Display, Colour, Foreground);
   m_SpaceSymbol = GetNumberSymbols() - 1;
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
-void CAlphabet::AddControlSymbol(const string NewCharacter, const string Display, int Colour, const string Foreground) {
+void CAlphabet::AddControlSymbol(const std::string NewCharacter, const std::string Display, int Colour, const std::string Foreground) {
   AddChar(NewCharacter, Display, Colour, Foreground);
   m_ControlSymbol = GetNumberSymbols() - 1;
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
-void CAlphabet::AddStartConversionSymbol(const string NewCharacter, const string Display, int Colour, const string Foreground) {
+void CAlphabet::AddStartConversionSymbol(const std::string NewCharacter, const std::string Display, int Colour, const std::string Foreground) {
   AddChar(NewCharacter, Display, Colour, Foreground);
   m_StartConversionSymbol = GetNumberSymbols() - 1;
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
-void CAlphabet::AddEndConversionSymbol(const string NewCharacter, const string Display, int Colour, const string Foreground) {
+void CAlphabet::AddEndConversionSymbol(const std::string NewCharacter, const std::string Display, int Colour, const std::string Foreground) {
   AddChar(NewCharacter, Display, Colour, Foreground);
   m_EndConversionSymbol = GetNumberSymbols() - 1;
 }
 
+void CAlphabet::AddResetSymbol(const std::string NewCharacter, const std::string Display, int Colour, const std::string Foreground) {
+  AddChar(NewCharacter, Display, Colour, Foreground);
+  m_iResetSymbol = GetNumberSymbols() - 1;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // diagnostic dump of character set
 
