@@ -25,6 +25,7 @@
 #include "config.h"
 #endif
 
+#include <climits>
 #include <deque>
 #include <cmath>
 #include <vector>
@@ -71,16 +72,18 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   ///
   /// Update the root location - called in response to regular timer
-  /// callbacks
+  /// callbacks, mainly from the tier events of various button
+  /// handlers
+  /// TODO: Make this a bit more central in the button handler hierarchy
   ///
   bool UpdatePosition(myint, myint, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded = NULL, int* pNumDeleted = NULL);  
 
   ///
   /// Notify the framerate class that a new frame has occurred
+  /// Called from CDasherInterfaceBase::NewFrame
   ///
   void NewFrame(unsigned long Time);
 
-
   ///
   /// Apply an offset to the 'target' coordinates - implements the jumps in
   /// two button dynamic mode.
@@ -106,6 +109,8 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   ///
   /// Reset the framerate class
+  /// TODO: Need to check semantics here
+  /// Called from CDasherInterfaceBase::UnPause;
   ///
 
   void Reset_framerate(unsigned long Time) {
@@ -116,6 +121,7 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   /// Initialise the framerate class - presumably called whenever
   /// dasher is stoppe, but the actual semantics here need to be
   /// verified
+  /// Called from CDasherInterfacebase::Halt 
   ///
 
   void Halt() {
@@ -127,16 +133,9 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   /// - could implement through the event subsystem instead
   ///
   
-  void SetBitrate(double TargetRate) {
-    m_fr.SetBitrate(TargetRate);
-  } 
-
-  ///
-  /// Clear the queue of old roots - used when those nodes become
-  /// invalid, eg during changes to conrol mode
-  ///
-
-  void ClearRootQueue();
+/*   void SetBitrate(double TargetRate) { */
+/*     m_fr.SetBitrate(TargetRate); */
+/*   }  */
 
   ///
   /// Reset counter of total nats entered
@@ -218,6 +217,7 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   ///
   /// Check whether a change of root node is needed, and perform the
   /// update if so
+  /// TODO: Could be done in UpdatePosition?
   ///
 
   bool CheckForNewRoot(CDasherView *pView);
@@ -240,55 +240,97 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   };
 
   void SetControlOffset(int iOffset);
- protected:
-  int m_iRenderCount;
-
 
  private:
 
-  CDasherInterfaceBase * m_pDasherInterface;
-
-  /////////////////////////////////////////////////////////////////////////////
-
-  // Interfaces
-
-  //  CLanguageModel *m_pLanguageModel;     // pointer to the language model
-
-  CLanguageModel::Context LearnContext; // Used to add data to model as it is entered
-
-  /////////////////////////////////////////////////////////////////////////////
+  /// Struct representing intermediate stages in the goto queue
+  ///
+  struct SGotoItem {
+    myint iN1;
+    myint iN2;
+    int iStyle;
+  };
+  
+  // Pointers to various auxilliary objects
+  CDasherInterfaceBase *m_pDasherInterface;
+  CNodeCreationManager *m_pNodeCreationManager;
 
+  // The root of the Dasher tree
   CDasherNode *m_Root;
 
   // Old root notes
+  // TODO: This should probably be rethought at some point - it doesn't really make a lot of sense
   std::deque < CDasherNode * >oldroots;
 
   // Rootmin and Rootmax specify the position of the root node in Dasher coords
-  myint m_Rootmin, m_Rootmax;
-  
-  myint m_iTargetMin;
-  myint m_iTargetMax;
+  myint m_Rootmin;
+  myint m_Rootmax;
 
-  myint m_iTargetOffset; // Displayed rootmin/max - actual rootmin/rootmax
-
-  myint m_Rootmin_min, m_Rootmax_max;
+  // Permitted range for root node - model cannot zoom beyond this
+  // point without falling back to a new root node.
+  myint m_Rootmin_min;
+  myint m_Rootmax_max;
 
+  // TODO: Does this need to be brought back? Make it relative to visible region?
   // The active interval over which Dasher nodes are maintained - this is most likely bigger than (0,DasherY)
-  //  CRange m_Active;
+  // CRange m_Active;
 
-  CFrameRate m_fr;              // keep track of framerate
+  // Offset used when presenting the model to the user, specified as
+  // Displayed rootmin/max - actual rootmin/rootmax
+  myint m_iTargetOffset; 
 
-  double m_dTotalNats;            // Information entered so far
+  CDasherNode *m_pLastOutput;
 
-  // the probability that gets added to every symbol
+  // Queue of goto locations (eg for button mode)
+  std::deque<SGotoItem> m_deGotoQueue;
+
+  /// TODO: Not sure what this actually does
   double m_dAddProb;
 
+  // Model parameters... (cached from settings store)
+
+  // Current maximum bitrate (ie zoom at far rhs).
   double m_dMaxRate;
 
-  int m_Stepnum;
+  // Whether game mode is active
+  // TODO: This isn't very functional at the moment
+  bool m_bGameMode;
+
+  // Whether characters entered by alphabet manager are expected to
+  // require conversion.
+  // TODO: Need to rethink this at some point.
+  bool m_bRequireConversion;
+
+  // Model status...
+
+  // Helper class to estimate frame rate
+  CFrameRate m_fr;  
+
+  // Time at which the model was started (ie last unpaused, used for gradual speed up)
+  // TODO: Implementation is very hacky at the moment
+  // TODO: Duplicates functionality previously implemented elsewhere
+  unsigned long m_iStartTime;
+  
+  // Offset into buffer of node currently under crosshair
+  int m_iOffset;
+
+  // Debug/performance information...
+
+  // Information entered so far in this model
+  double m_dTotalNats; 
+
+  // Number of nodes rendered
+  int m_iRenderCount;
+
+
 
   CDasherNode *Get_node_under_mouse(myint smousex, myint smousey);
 
+  ///
+  /// Go directly to a given coordinate - check semantics
+  ///
+  void NewGoTo(myint n1, myint n2, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted);
+
   ///
   /// CDasherModel::Get_new_root_coords( myint Mousex,myint Mousey )
   /// 
@@ -303,14 +345,24 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   ///
   void Get_new_root_coords(myint mousex, myint mousey, myint &iNewMin, myint &iNewMax, unsigned long iTime);
 
+
+  /// Called from ScheduleZoom
   double CorrectionFactor(int dasherx, int dashery);
 
-  void DeleteTree();
-
+  /// Should be public?
   void InitialiseAtOffset(int iOffset, CDasherView *pView);
 
+  /// Called from InitialiseAtOffset
+  void DeleteTree();
+
+  /// Push nodes recursively (d'oh!) - seems to be a little broken at the moment, need to check where this gets called
+  /// TODO: Need to sort out, need to check that this isn't generating too many nodes
   void Recursive_Push_Node(CDasherNode * pNode, int depth);
 
+  /// Create the children of a Dasher node
+  void Push_Node(CDasherNode * pNode); 
+
+
   ///
   /// Perform output on a node - recurse up the tree outputting any
   /// symbols which have not been output so far. Neeed to check
@@ -320,12 +372,6 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void RecursiveOutput(CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded);
 
 
-  ///
-  /// Go directly to a given coordinate - check semantics
-  ///
-  
-  void NewGoTo(myint n1, myint n2, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted);
-
   ///
   /// Check semantics here
   ///
@@ -333,12 +379,6 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void OutputCharacters(CDasherNode * node);
   bool DeleteCharacters(CDasherNode * newnode, CDasherNode * oldnode, int* pNumDeleted = NULL);
 
-  ///
-  /// Create the children of a Dasher node
-  ///
-
-  void Push_Node(CDasherNode * pNode); 
-
   /// 
   /// Old style drilling down of nodes - optionally can still be
   /// called
@@ -389,26 +429,13 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   bool RecursiveCheckRoot(CDasherNode *pNode, CDasherNode **pNewNode, bool &bFound);
 
-  CNodeCreationManager *m_pNodeCreationManager;
-  bool m_bGameMode;
 
+  ///
+  /// Clear the queue of old roots - used when those nodes become
+  /// invalid, eg during changes to conrol mode
+  ///
 
-  unsigned long m_iStartTime;
-
-  struct SGotoItem {
-    myint iN1;
-    myint iN2;
-    int iStyle;
-  };
-
-  std::deque<SGotoItem> m_deGotoQueue;
-  
-  // Whether characters entered by alphabet manager are expected to
-  // require conversion.
-  // TODO: Need to rethink this at some point.
-  bool m_bRequireConversion;
-
-  int m_iOffset;
+  void ClearRootQueue();
 
 };
 /// @}
