@@ -5,11 +5,16 @@
 
 #include "LanguageModelling/LanguageModel.h"
 #include "LanguageModelling/PPMLanguageModel.h"
-#include <ChiCEInterface.h>
-#include <iostream>
-#include <fstream>
 #include "PinYinConversionHelper.h"
 
+
+#include "SCENode.h"
+
+#include <fstream>
+#include <iostream>
+
+
+
 using namespace Dasher;
 
 CPinYinConversionHelper::CPinYinConversionHelper(Dasher::CEventHandler *pEventHandler, CSettingsStore *pSettingsStore, Dasher::CAlphIO *pCAlphIO){
@@ -57,31 +62,99 @@ CPinYinConversionHelper::CPinYinConversionHelper(Dasher::CEventHandler *pEventHa
 
 
   BuildDataBase();
-  CEInitialise();
 
+  pParser = new CPinyinParser();
 }
   
-bool CPinYinConversionHelper::Convert(const std::string &strSource, SCENode ** pRoot, int * HZCount, int CMid) {
+bool CPinYinConversionHelper::Convert(const std::string &strSource, SCENode ** pRoot, int * childCount, int CMid) {
 
-  SCENode * pStart;
+  SCENode *pConversionList;
+  int iHZCount;
 
-  if(CEConvert (strSource.c_str(), &pStart, HZCount, CMid)){ 
-    *pRoot= pStart;
+  // TODO: Reimplement
+  //  if(CEConvert (strSource.c_str(), &pConversionList, &iHZCount, CMid)){ 
 
-    // Connect up the rest of the nodes to make a lattice
-    SCENode *pHead(pStart);
+  if(pParser && pParser->Convert(strSource, pRoot)) {
+//     SCENode *pHead(pConversionList);
+
+//     std::vector<SCENode *> vHeads;
+
+//     while(pHead) {
+//       vHeads.push_back(pHead);
+//       pHead = pHead->pChild;
+//     }
+
+//     SCENode *pTail = NULL;
+//     SCENode *pNextTail = NULL;
+
+//     for(std::vector<SCENode *>::reverse_iterator it(vHeads.rbegin()); it != vHeads.rend(); ++it) {
+//       SCENode *pCurrentNode(*it);
+
+//       SCENode *pPreviousNode = NULL;
+
+//       while(pCurrentNode) {
+// 	SCENode *pNewNode = new SCENode;
+
+// 	pNewNode->pszConversion = pCurrentNode->pszConversion;
+// 	pNewNode->IsHeadAndCandNum = pCurrentNode->IsHeadAndCandNum;
+// 	pNewNode->CandIndex = pCurrentNode->CandIndex;
+// 	pNewNode->Symbol = pCurrentNode->Symbol;
+// 	//	pNewNode->SumPYProbStore = pCurrentNode->SumPYProbStore;
+// 	pNewNode->IsComplete = pCurrentNode->IsComplete;
+// 	pNewNode->AcCharCount = pCurrentNode->AcCharCount;  /*accumulative character count*/
+// 	pNewNode->NodeSize = pCurrentNode->NodeSize;
+// 	pNewNode->HZFreq = pCurrentNode->HZFreq;
+// 	pNewNode->HZProb = pCurrentNode->HZProb;
+	
+// 	if(pTail)
+// 	  pNewNode->SetChild(pTail);
+
+// 	if(pPreviousNode) {
+// 	  pPreviousNode->SetNext(pNewNode);
+// 	}
+// 	else {
+// 	  pNextTail = pNewNode;
+// 	  pNextTail->Ref();
+// 	}
+
+// 	if(pPreviousNode)
+// 	  pPreviousNode->Unref();
+
+// 	pPreviousNode = pNewNode;
+// 	pCurrentNode = pCurrentNode->pNext;
+//       }
+
+//       if(pPreviousNode)
+// 	pPreviousNode->Unref();
+
+//       if(pTail)
+// 	pTail->Unref();
+
+//       pTail = pNextTail;
+  
+//     }
+
+//     *pRoot = pTail;
+
+
+// //     // TODO: Now need to convert...
+
+// //     *pRoot= pStart;
+// 2
+// //     // Connect up the rest of the nodes to make a lattice
+// //     SCENode *pHead(pStart);
     
-    while(pHead) {
-      SCENode *pNewChild(pHead->pChild);
-      SCENode *pCurrent(pHead->pNext);
+// //     while(pHead) {
+// //       SCENode *pNewChild(pHead->GetChild());
+// //       SCENode *pCurrent(pHead->GetNext());
       
-      while(pCurrent) {
-	pCurrent->pChild = pNewChild;
-	pCurrent = pCurrent->pNext;
-      }
+// //       while(pCurrent) {
+// // 	pCurrent->SetChild(pNewChild);
+// // 	pCurrent = pCurrent->GetNext();
+// //       }
       
-      pHead = pHead->pChild;
-    }
+// //       pHead = pHead->GetChild();
+// //     }
    
     return 1;
   }
@@ -91,9 +164,45 @@ bool CPinYinConversionHelper::Convert(const std::string &strSource, SCENode ** p
   }
 }
 
-void CPinYinConversionHelper::AssignSizes(SCENode * pStart, Dasher::CLanguageModel::Context context, long normalization, int uniform, int iNChildren){
+unsigned int CPinYinConversionHelper::GetSumPYProbs(Dasher::CLanguageModel::Context context, SCENode * pPYCandStart, int norm){
 
-  SCENode *pNode = pStart;
+  std::vector <unsigned int > Probs;
+  unsigned int sumProb=0;
+  
+  m_pLanguageModel->GetProbs(context, Probs, norm, 0);
+
+  SCENode * pCurrentNode = pPYCandStart;
+
+  while(pCurrentNode){
+    
+    std::vector <symbol >Symbols;
+    std::string HZ = static_cast<std::string>(pCurrentNode->pszConversion);
+    // Distribute the remaining space evenly
+    
+    m_pAlphabet->GetSymbols(&Symbols, &HZ, 0);    
+
+    if(Symbols.size()!=0)
+      sumProb += Probs[Symbols[0]];
+    pCurrentNode = pCurrentNode->GetNext();
+
+  }
+
+  return sumProb;
+}
+
+void CPinYinConversionHelper::GetProbs(Dasher::CLanguageModel::Context context, std::vector < unsigned int >&Probs, int norm){
+}
+  
+  
+
+
+void CPinYinConversionHelper::AssignSizes(SCENode **pStart, Dasher::CLanguageModel::Context context, long normalization, int uniform, int iNChildren){
+
+  //  std::cout << "Assigning sizes: " << *pStart << " (" << (*pStart)->pszConversion << ")" << std::endl;
+
+  SCENode *pNewStart = *pStart;
+
+  SCENode *pNode = pNewStart;
 
   std::vector <unsigned int > Probs;
 
@@ -121,8 +230,23 @@ void CPinYinConversionHelper::AssignSizes(SCENode * pStart, Dasher::CLanguageMod
     }
     */
 
-  // CLanguageModel::Context context = m_pLanguageModel->CreateEmptyContext();
-  m_pLanguageModel->GetProbs(context, Probs, nonuniform_norm);
+
+  //  context = m_pLanguageModel->CreateEmptyContext();
+
+   //Testing Code for PYCHelper GetPYSumProbs
+ 
+  /*
+ CLanguageModel::Context iContext = m_pLanguageModel->CreateEmptyContext();
+
+  SCENode * pTemp = pStart;
+  while(pTemp){
+    std::cout<<"test sum probs"<<GetSumPYProbs(iContext, pTemp, nonuniform_norm)<<std::endl;
+    std::cout<<"test norm"<<nonuniform_norm<<std::endl;
+    pTemp=pTemp->pChild;
+  }
+  */
+
+  m_pLanguageModel->GetProbs(context, Probs, nonuniform_norm, 0);
 
   /*  
   std::vector<unsigned int>::iterator it;
@@ -135,39 +259,78 @@ void CPinYinConversionHelper::AssignSizes(SCENode * pStart, Dasher::CLanguageMod
   //  unsigned int sum; 
 
 
-  unsigned int sumProb=0;
+  unsigned long long int sumProb=0;
 
+  std::vector <symbol >Symbols;
+  std::string HZ;
+  CLanguageModel::Context iCurrentContext;
+
+
+  //std::cout<<"start"<<std::endl;
   while(pNode){
 
-    std::vector <symbol >Symbols;
-    std::string HZ = static_cast<std::string>(pNode->pszConversion);
-    // Distribute the remaining space evenly
-    
-    m_pAlphabet->GetSymbols(&Symbols, &HZ, 0);    
-
-    if(Symbols.size()!=0)
-      sumProb += Probs[Symbols[0]];
-    pNode = pNode->pNext;
-  }
-  
-  pNode = pStart;
-  while(pNode){
-    //    std::cout<<"HZ"<<HZ<<std::endl;
-    std::vector <symbol >Symbols;
-    std::string HZ = static_cast<std::string>(pNode->pszConversion);
-    // Distribute the remaining space evenly
-    
+    Symbols.clear();
+    HZ = static_cast<std::string>(pNode->pszConversion);
     m_pAlphabet->GetSymbols(&Symbols, &HZ, 0);    
 
     if(Symbols.size()!=0){
-      if(sumProb!=0)
-	pNode->NodeSize =Probs[Symbols[0]]*normalization/sumProb;
-      
       pNode->Symbol = Symbols[0];
+      //sumProb += Probs[Symbols[0]];
+
+      
+      iCurrentContext=m_pLanguageModel->CloneContext(context);
+      m_pLanguageModel->EnterSymbol(iCurrentContext, pNode->Symbol);
+      
+      if(pNewStart->GetChild()){
+	pNode->SumPYProbStore = GetSumPYProbs(iCurrentContext, pNewStart->GetChild(), nonuniform_norm);
+	//std::cout<<"sumpyprobstore"<<pNode->SumPYProbStore<<std::endl;
+      }
+      else
+	pNode->SumPYProbStore = 1;
+	
+      sumProb += (Probs[pNode->Symbol]*(pNode->SumPYProbStore));
+      //std::cout<<"Probs[symbol]"<<Probs[Symbols[0]]<<std::endl;
+      //std::cout<<"sumProbs"<<sumProb<<std::endl;
+    }
+    else
+      pNode->Symbol = -1;
+
+    pNode = pNode->GetNext();
+  }
+
+
+
+  pNode = pNewStart;
+  while(pNode){
+    /*
+    std::vector <symbol >Symbols;
+    std::string HZ = static_cast<std::string>(pNode->pszConversion);
+ 
+    
+    m_pAlphabet->GetSymbols(&Symbols, &HZ, 0);    
+    */
+
+    if(pNode->Symbol!=-1){
+      if(sumProb!=0){
+
+	//	iCurrentContext=m_pLanguageModel->CloneContext(context);
+	//m_pLanguageModel->EnterSymbol(iCurrentContext, pNode->Symbol);
+      
+	
+	pNode->NodeSize =static_cast<unsigned long long int>(Probs[pNode->Symbol])*(pNode->SumPYProbStore)*normalization/sumProb;
+
+	/*
+	std::cout<<"HZ"<<pNode->pszConversion<<std::endl;
+	std::cout<<"Probs"<<Probs[pNode->Symbol]<<std::endl;
+	std::cout<<"SumProbStore"<<pNode->SumPYProbStore<<std::endl;
+	std::cout<<"above"<<Probs[pNode->Symbol]*(pNode->SumPYProbStore)<<std::endl;
+	std::cout<<"sumprob"<<sumProb<<std::endl;
+	std::cout<<"nodesize"<<pNode->NodeSize<<std::endl;
+	*/
+      }
     }
     else{
       pNode->NodeSize = 0;//hopefully this will be not be displayed
-      pNode->Symbol = -1;
     }
 
     if(pNode->NodeSize < 1)
@@ -175,10 +338,10 @@ void CPinYinConversionHelper::AssignSizes(SCENode * pStart, Dasher::CLanguageMod
     
     iRemaining -= pNode->NodeSize; 
 
-    pNode = pNode->pNext;
+    pNode = pNode->GetNext();
   }
 
-  pNode = pStart;
+  pNode = pNewStart;
 
   while(pNode){
     
@@ -188,9 +351,15 @@ void CPinYinConversionHelper::AssignSizes(SCENode * pStart, Dasher::CLanguageMod
     
     iRemaining -= iDiff;
     --iLeft;
+    pNode = pNode->GetNext();
+  }
+  /*
+  pNode = pStart;
+  while(pNode){
+    std::cout<<"size"<<pNode->NodeSize<<std::endl;
     pNode = pNode ->pNext;
   }
-
+  */
   /*
   pNode = pStart;
   while(pNode){
@@ -203,8 +372,6 @@ void CPinYinConversionHelper::AssignSizes(SCENode * pStart, Dasher::CLanguageMod
 
   //std::cout<<catStr<<std::endl;
   
-  
-
   //for(int i=0; i<Symbols.size(); i++)
   //  std::cout<<Symbols[i]<<",";
 
@@ -220,18 +387,20 @@ void CPinYinConversionHelper::AssignSizes(SCENode * pStart, Dasher::CLanguageMod
 }
 
 bool CPinYinConversionHelper::GetPhraseList(int HZIndex, SCENode ** psOutput, int CMid){
-  SCENode * pStart;
+//   SCENode * pStart;
   
-  if(CEGetPhraseList(HZIndex, &pStart, CMid)){ 
+//   if(CEGetPhraseList(HZIndex, &pStart, CMid)){ 
    
-    *psOutput= pStart;
+//     *psOutput= pStart;
    
-    return 1;
-  }
-  else{
-    *psOutput = 0;
-    return 0;
-  }
+//     return 1;
+//   }
+//   else{
+//     *psOutput = 0;
+//     return 0;
+//   }
+
+  return 0;
 }
 
 void CPinYinConversionHelper::BuildDataBase(){
@@ -316,7 +485,7 @@ void CPinYinConversionHelper::ProcessPhrase(HZIDX HZIndex){
     return;
 
   if(pPhraseList->AcCharCount>4)
-    pNode = pPhraseList->pNext;
+    pNode = pPhraseList->GetNext();
   else 
     pNode = pPhraseList;
 
@@ -381,7 +550,7 @@ void CPinYinConversionHelper::ProcessPhrase(HZIDX HZIndex){
       }
     }
 
-    pNode = pNode ->pNext;
+    pNode = pNode ->GetNext();
     iIdx ++;
   }
   m_bPhrasesProcessed[HZIndex]=1;
@@ -395,7 +564,7 @@ CANDIDX CPinYinConversionHelper::HZLookup(HZIDX HZIndex, const std::string &strS
   // this was done before candindex was put into node member, change
   // if have time
 
-  int iIdx(0);
+  //  int iIdx(0);
   if((HZIndex > m_iHZCount-1)||strSource.size()!=3)
     return -1;
 
@@ -515,7 +684,8 @@ int CPinYinConversionHelper::CalculateScore(CDasherNode * pNode, CANDIDX CandInd
 
 void CPinYinConversionHelper::TrainChPPM(){
 
-  for(int i =0; i<15;i++)
+  // TODO: Changed to 1 from 10
+  for(int i =0; i<1;i++)
     ProcessFile(i);
 
 }
@@ -534,7 +704,7 @@ void CPinYinConversionHelper::ProcessFile(int index){
  
   char str[4];
   std::string HZ;
-  int  i, j, iLen;
+  //  int  i, j, iLen;
 
 
 
