@@ -1,26 +1,29 @@
 #ifndef __PINYIN_CONVERSION_HELPER_H__
 #define __PINYIN_CONVERSION_HELPER_H__
 
-#include "Alphabet/Alphabet.h"
 #include "Alphabet/AlphIO.h"
-#include "LanguageModelling/LanguageModel.h"
+#include "Alphabet/Alphabet.h"
 #include "ConversionHelper.h"
 #include "DasherNode.h"
+#include "LanguageModelling/LanguageModel.h"
+#include "PinyinParser.h"
 
 //both of these start from 0
 typedef int HZIDX; 
 typedef int CANDIDX; 
 
-/// \ingroup Model
-/// @{
 class CPinYinConversionHelper : public CConversionHelper {
  public:
 
   CPinYinConversionHelper(Dasher::CEventHandler *pEventHandler,  CSettingsStore *pSettingsStore, Dasher::CAlphIO *pAlphIO);
   
-  virtual bool Convert(const std::string &strSource, SCENode ** pRoot, int * HZCount, int CMid);
+  virtual bool Convert(const std::string &strSource, SCENode ** pRoot, int * childCount, int CMid);
 
-  virtual void AssignSizes(SCENode * pStart, Dasher::CLanguageModel::Context context , long normalization, int uniform, int iNChildren);
+  virtual void AssignSizes(SCENode **pStart, Dasher::CLanguageModel::Context context , long normalization, int uniform, int iNChildren);
+
+  virtual void GetProbs(Dasher::CLanguageModel::Context context, std::vector < unsigned int >&Probs, int norm);
+
+  virtual unsigned int GetSumPYProbs(Dasher::CLanguageModel::Context context, SCENode * pPYCandStart,int norm);
 
   virtual Dasher::CLanguageModel *  GetLanguageModel(){
     return m_pLanguageModel;
@@ -76,6 +79,8 @@ class CPinYinConversionHelper : public CConversionHelper {
                            //CalculateScore
 
 
+  CPinyinParser *pParser;
+
 };
-/// @}
+
 #endif
