@@ -1,9 +1,6 @@
 #ifndef __dasher_main_h__
 #define __dasher_main_h__
 
-#include "DasherAppSettings.h"
-#include "dasher_editor.h"
-
 #include <glib.h>
 #include <glib-object.h>
 
@@ -34,11 +31,12 @@ typedef struct _SCommandLine SCommandLine;
 struct _SCommandLine {
   gchar *szFilename;
   gchar *szAppStyle;
+  gchar *szOptions;
 };
 
 DasherMain *dasher_main_new(int *argc, char ***argv, SCommandLine *pCommandLine);
 GType dasher_main_get_type();
-//DasherEditor *dasher_main_get_editor(DasherMain *pSelf);
+//DasherEditorInternal *dasher_main_get_editor(DasherMain *pSelf);
 void dasher_main_show(DasherMain *pSelf);
 G_END_DECLS
 
